'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _d = require('d3');

var _d2 = _interopRequireDefault(_d);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _class = function _class(config) {
  (0, _classCallCheck3.default)(this, _class);
  var container = config.container,
      radiusMaker = config.radiusMaker,
      data = config.data,
      name = config.name,
      value = config.value;


  var color = ['#f2c3c8', '#c5cae9', '#a9d4d0', '#f2edba', '#ecb2c6', '#b2d3ef', '#bedbbf', '#f0e3b1', '#d6b5dc', '#aadaf0', '#d1e1be', '#f2d5a9', '#c7badd', '#a9dfe6', '#e4e8b9', '#f2c2b3'];

  var minRadius = 15;
  var maxRadius = 80;
  // separation between nodes
  var padding = 6;
  var width = container.clientWidth;
  var height = container.clientHeight || width * .5;

  var radiusMakerRange = _d2.default.extent(data, function (d) {
    return d[radiusMaker];
  });

  var radiusDomain = _d2.default.scale.linear().range([minRadius, maxRadius]).domain(radiusMakerRange);

  var fontSizeDomain = _d2.default.scale.linear().range([10, 30]).domain(radiusMakerRange);

  data.forEach(function (item) {
    item.radius = radiusDomain(item[radiusMaker]);
    item.fontSize = fontSizeDomain(item[radiusMaker]);
    item.cx = Math.floor(Math.random() * width);
    item.cy = height / 2;
  });

  var svg = _d2.default.select(container).append('svg').attr('width', width).attr('height', height);

  // Move nodes toward cluster focus.
  var gravity = function gravity(alpha) {
    return function (d) {
      d.x += (d.cx - d.x) * alpha;
      d.y += (d.cy - d.y) * alpha;
    };
  };

  // Resolve collisions between nodes.
  var collide = function collide(alpha) {
    var quadtree = _d2.default.geom.quadtree(data);
    return function (d) {
      var r = d.radius + maxRadius + padding;
      var nx1 = d.x - r;
      var nx2 = d.x + r;
      var ny1 = d.y - r;
      var ny2 = d.y + r;
      quadtree.visit(function (quad, x1, y1, x2, y2) {
        if (quad.point && quad.point !== d) {
          var x = d.x - quad.point.x;
          var y = d.y - quad.point.y;
          var l = Math.sqrt(x * x + y * y);
          var _r = d.radius + quad.point.radius + (d.color !== quad.point.color) * padding;
          if (l < _r) {
            l = (l - _r) / l * alpha;
            d.x -= x *= l;
            d.y -= y *= l;
            quad.point.x += x;
            quad.point.y += y;
          }
        }
        return x1 > nx2 || x2 < nx1 || y1 > ny2 || y2 < ny1;
      });
    };
  };

  var forceLayout = _d2.default.layout.force().nodes(data).size([width, height]).gravity(.2).charge(-300).on('tick', function (e) {
    bubbles.each(gravity(.2 * e.alpha)).each(collide(.5)).attr('transform', function (d) {
      return 'translate(' + d.x + ', ' + d.y + ')';
    });
  }).start();

  var bubbles = svg.selectAll('g').data(data).enter().append('g').on('mouseenter', function (d) {
    // self.popover.triggerElement && self.popover.triggerElement.off('mouseleave');
    // self.popover.triggerElement = $(d3.event.target);
    // self.tooltip.model.reset(d);
    _d2.default.select(this).select('circle').transition().ease(_d2.default.ease('back')).duration(200).attr('r', d.radius * 1.3);
  }).on('mouseleave', function (d) {
    _d2.default.select(this).select('circle').transition().ease(_d2.default.ease('back')).duration(200).attr('r', d.radius);
  }).call(forceLayout.drag);

  bubbles.append('circle').attr('r', function (d) {
    return d.radius;
  }).attr('stroke', function (d, i) {
    return color[i % color.length];
  }).attr('fill', function (d, i) {
    return color[i % color.length];
  }).attr('fill-opacity', .4);

  var textContainers = bubbles.append('text').attr('text-anchor', 'middle').style('font-size', function (d) {
    return d.fontSize;
  }).attr('fill', '#807778');

  textContainers.append('tspan').attr('x', 0).attr('y', '-.2em').text(function (d) {
    return d[name];
  });

  var valueFormat = _d2.default.format(value.format || '.2s');

  textContainers.append('tspan').attr('x', 0).attr('y', '1em').text(function (d) {
    return valueFormat(d[value.key]);
  });
}; /**
    * Copyright 2016-present, Baifendian, Inc.
    * All rights reserved.
    *
    * This source code is licensed under the BSD-style license found in the
    * LICENSE file in the root directory of this source tree. An additional grant
    * of patent rights can be found in the PATENTS file in the same directory.
    */

exports.default = _class;