'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _objectWithoutProperties2 = require('babel-runtime/helpers/objectWithoutProperties');

var _objectWithoutProperties3 = _interopRequireDefault(_objectWithoutProperties2);

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _reactDom = require('react-dom');

var _reactDom2 = _interopRequireDefault(_reactDom);

var _classlist = require('classlist');

var _classlist2 = _interopRequireDefault(_classlist);

var _ToggleNode = require('../_shared/ToggleNode');

var _ToggleNode2 = _interopRequireDefault(_ToggleNode);

var _EventPool = require('../_shared/EventPool');

var _EventPool2 = _interopRequireDefault(_EventPool);

var _ModalContent = require('./ModalContent');

var _ModalContent2 = _interopRequireDefault(_ModalContent);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Copyright 2016-present, Baifendian, Inc.
 * All rights reserved.
 *
 * This source code is licensed under the BSD-style license found in the
 * LICENSE file in the root directory of this source tree. An additional grant
 * of patent rights can be found in the PATENTS file in the same directory.
 */

var scrollbarWidth = function () {
  var scrollDiv = document.createElement('div');
  var body = document.body;

  scrollDiv.className = 'bfd-modal--scrollbar-measure';
  body.appendChild(scrollDiv);
  var scrollbarWidth = scrollDiv.offsetWidth - scrollDiv.clientWidth;
  body.removeChild(scrollDiv);

  return scrollbarWidth;
}();

var Modal = function (_Component) {
  (0, _inherits3.default)(Modal, _Component);

  function Modal(props) {
    (0, _classCallCheck3.default)(this, Modal);

    var _this = (0, _possibleConstructorReturn3.default)(this, (Modal.__proto__ || (0, _getPrototypeOf2.default)(Modal)).call(this, props));

    _this.closeCallbacks = new _EventPool2.default();
    _this.state = {
      open: props.open || false
    };
    return _this;
  }

  (0, _createClass3.default)(Modal, [{
    key: 'componentWillReceiveProps',
    value: function componentWillReceiveProps(nextProps) {
      'open' in nextProps && this.setState({ open: nextProps.open });
    }
  }, {
    key: 'shouldComponentUpdate',
    value: function shouldComponentUpdate(nextProps, nextState) {
      return !(this.state.open === nextState.open && nextState.open === false);
    }
  }, {
    key: 'componentDidMount',
    value: function componentDidMount() {
      this.updateBodyState(this.state.open, false);
      if (this.state.open) {
        this.renderIntoDocument();
      }
    }
  }, {
    key: 'componentDidUpdate',
    value: function componentDidUpdate(prevProps, prevState) {
      this.updateBodyState(this.state.open, prevState.open);
      if (this.state.open) {
        this.renderIntoDocument();
      } else {
        prevState.open && this.toggleModal.close();
        var modalContent = this.context.modalContent;

        if (modalContent) {
          modalContent.goForward();
        }
      }
    }
  }, {
    key: 'componentWillUnmount',
    value: function componentWillUnmount() {
      if (this.containerNode) {
        _reactDom2.default.unmountComponentAtNode(this.containerNode);
        document.body.removeChild(this.containerNode);
      }
    }
  }, {
    key: 'updateBodyState',
    value: function updateBodyState(open, prevOpen) {
      if (this.context.modalContent) {
        return;
      }
      var body = document.body;
      var bodyPaddingRight = parseInt(body.style.paddingRight, 10) || 0;
      var _scrollbarWidth = body.scrollHeight > window.innerHeight ? scrollbarWidth : 0;
      if (open && !prevOpen) {
        (0, _classlist2.default)(body).add('bfd-modal--open');
        body.style.paddingRight = bodyPaddingRight + _scrollbarWidth + 'px';
      } else if (!open && prevOpen) {
        this.closeCallbacks.add(function () {
          (0, _classlist2.default)(body).remove('bfd-modal--open');
          if (bodyPaddingRight) {
            body.style.paddingRight = bodyPaddingRight - _scrollbarWidth + 'px';
          } else {
            body.style.paddingRight = '';
          }
        });
      }
    }

    /**
     * @public
     * @name this.refs.modal.open
     * @description 打开模态框
     */

  }, {
    key: 'open',
    value: function open() {
      this.setState({ open: true });
      this.props.onToggle && this.props.onToggle(true);
    }

    /**
     * @public
     * @name this.refs.modal.close
     * @param {function} [callback] 关闭后的回调，动画结束后执行
     * @description 关闭模态框
     */

  }, {
    key: 'close',
    value: function close() {
      var callback = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : this.props.onClose;

      this.setState({ open: false });
      this.props.onToggle && this.props.onToggle(false);
      this.closeCallbacks.add(callback);
    }
  }, {
    key: 'renderIntoDocument',
    value: function renderIntoDocument() {
      var _this2 = this;

      if (!this.containerNode) {
        this.containerNode = document.createElement('div');
        document.body.appendChild(this.containerNode);
      }
      var onRendered = function onRendered() {
        if (!_this2.toggleModal) {
          var node = _reactDom2.default.findDOMNode(_this2.content);
          _this2.toggleModal = new _ToggleNode2.default(node, 'bfd-modal--open');
          _this2.toggleModal.onClose = function () {
            return _this2.closeCallbacks.free();
          };
        }
        _this2.toggleModal.open();
        var modalContent = _this2.context.modalContent;

        if (modalContent) {
          modalContent.backUp();
        }
      };
      this.renderIntoDocument = function () {
        var _props = _this2.props,
            open = _props.open,
            onToggle = _props.onToggle,
            onClose = _props.onClose,
            other = (0, _objectWithoutProperties3.default)(_props, ['open', 'onToggle', 'onClose']);

        _reactDom2.default.render(_react2.default.createElement(_ModalContent2.default, (0, _extends3.default)({
          ref: function ref(content) {
            return _this2.content = content;
          },
          close: _this2.close.bind(_this2),
          modal: _this2
        }, other)), _this2.containerNode, onRendered);
      };
      this.renderIntoDocument();
    }
  }, {
    key: 'render',
    value: function render() {
      return null;
    }
  }]);
  return Modal;
}(_react.Component);

Modal.contextTypes = {
  modalContent: _react.PropTypes.object
};

Modal.propTypes = {

  // 是否打开
  open: _react.PropTypes.bool,

  // 切换 open 状态后的回调，参数为切换后的 open 状态，立刻执行，不会等到动画结束后
  onToggle: _react.PropTypes.func,

  // 是否锁定，锁定后点击背景无法关闭
  lock: _react.PropTypes.bool,

  // 关闭后的回调，动画结束后执行。如果 close 方法传入回调，则此属性不会触发
  onClose: _react.PropTypes.func,

  // 尺寸，可选值：`sm`, `lg`, 默认中等尺寸
  size: _react.PropTypes.oneOf(['sm', 'lg'])
};

exports.default = Modal;