'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

var _d2 = require('d3');

var _d3 = _interopRequireDefault(_d2);

var _pie = require('./pie');

var _pie2 = _interopRequireDefault(_pie);

var _assembleTooltip = require('./assembleTooltip');

var _assembleTooltip2 = _interopRequireDefault(_assembleTooltip);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.default = function (env) {

  var legend = {
    layout: 'vertical',
    align: 'right',
    style: {
      x: 0,
      y: 0
    }
  };
  (0, _assign2.default)(legend, env.config.legend);

  var dom = _d3.default.select(env.container).append('div').attr('class', 'legend pie-legend').style('position', 'absolute').selectAll('div').data(env.config.data).enter().append('div').style('cursor', 'pointer').style('display', 'block');

  switch (legend.align) {
    case 'left':
      _d3.default.select(env.container).select('.legend').style({
        left: legend.style.x + 'px' || '0px',
        top: legend.style.y + 'px' || '0px'
      });
      break;
    case 'right':
      _d3.default.select(env.container).select('.legend').style({
        left: legend.style.x + env.width + 'px' || env.width,
        top: legend.style.y + 'px' || '0px'
      });
      break;
    case 'top':
      _d3.default.select(env.container).select('.legend').style({
        left: legend.style.x + 'px' || '0px',
        top: legend.style.y + 'px' || '0px'
      });
      if (legend.layout == 'horizontal') {
        _d3.default.select(env.container).selectAll('div').style({
          display: 'inline-block'
        });
      }
      break;
    case 'bottom':
      _d3.default.select(env.container).select('.legend').style({
        left: legend.style.x + 'px' || '0px',
        top: legend.style.y + env.height + 'px' || '0px'
      });
      if (legend.layout == 'horizontal') {
        _d3.default.select(env.container).selectAll('div').style({
          display: 'inline-block'
        });
      }
      break;
    default:
      break;
  }

  dom.on('click', function (d) {

    var node = _d3.default.select(this);

    var isDisabled = node.classed('disabled');

    node.classed('disabled', !isDisabled);

    // update env.config.data
    isDisabled ? env.config.dataLegend.push(d) : env.config.dataLegend = function () {
      var arr = [];
      env.config.dataLegend.map(function (_d) {
        if (_d.id !== d.id) {
          arr.push(_d);
        }
      });
      return arr;
    }();

    // clear pie/tooltip
    env.svg.select('.pie-slices').selectAll('path').remove();
    env.svg.select('.pie-labels').selectAll('text').remove();
    env.svg.select('.pie-lines').selectAll('polyline').remove();

    // draw pie repeat
    (0, _pie2.default)(env, false);

    (0, _assembleTooltip2.default)(env, false);
  }).each(function (d) {
    var node = _d3.default.select(this);
    node.append('span').attr('class', 'legend-rect').style('background-color', d.color);
    node.append('span').text(d.name);
  });
}; /**
    * Copyright 2016-present, Baifendian, Inc.
    * All rights reserved.
    *
    * This source code is licensed under the BSD-style license found in the
    * LICENSE file in the root directory of this source tree. An additional grant
    * of patent rights can be found in the PATENTS file in the same directory.
    */