'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

var _d = require('d3');

var _d2 = _interopRequireDefault(_d);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.default = function (env, flag) {

  var _config = {
    radius: {
      inner: 0.75
    },
    animation: {
      pie: 2500,
      lineText: 500
    }
  };

  (0, _assign2.default)(_config, env.config);

  var initialAnimDelay = 100;
  var arcAnimDelay = 120;
  var arcAnimDur = _config.animation.pie;
  var arcAnimLineText = _config.animation.lineText;
  var width = env.width;
  var height = env.height;
  var radius = Math.min(width, height) / 2;
  var arcs = [];

  // for drawing slices
  var arcOut = _d2.default.svg.arc().outerRadius(0.6 * radius + 4).innerRadius(0.6 * _config.radius.inner * radius - 4);
  arcs.push(arcOut);

  var arc = _d2.default.svg.arc().outerRadius(0.6 * radius).innerRadius(0.6 * _config.radius.inner * radius);
  arcs.push(arc);

  var arc2 = _d2.default.svg.arc().outerRadius(0.6 * _config.radius.inner * radius - 8).innerRadius(0.6 * _config.radius.inner * radius - 12);
  arcs.push(arc2);

  // for labels and polylines  设置label和line的位置。
  var outerArc = _d2.default.svg.arc().innerRadius(0.5 * radius).outerRadius(0.9 * radius);

  var pie = _d2.default.layout.pie().value(function (d) {
    return d.value;
  });

  /*
   *填充颜色并画圆弧，并设置动画效果。
   */
  // define slice 绑定数据
  var slice = env.svg.select('.pie-slices').datum(flag ? _config.data : _config.dataLegend).selectAll('path').data(pie);

  var _loop = function _loop(i) {

    slice.enter().append('path').attr('fill', function (d) {
      return d.data.color;
    }).attr('d', function (d) {
      return arcs[i](d);
    }).attr('transform', function () {
      return 'rotate(-180, 0, 0)';
    }).style('opacity', 0).transition().delay(function (d, i) {
      return i * arcAnimDelay + initialAnimDelay;
    }).duration(arcAnimDur).ease('elastic').style('opacity', 1.0).attr('transform', 'rotate(0,0,0)');

    if (i === 0) {
      slice.style('display', 'none').attr('class', 'bfd-pie-lg');
    }
    if (i === 1) {
      slice.attr('class', 'bfd-pie-md');
    }
    if (i === 2) {
      slice.style('display', 'none').attr('class', 'bfd-pie-flag');
    }
  };

  for (var i = 0; i < arcs.length; i++) {
    _loop(i);
  }

  // 如果饼图的宽度太小就不显示text和line。
  if (env.width < 500) return;

  // 设置text 
  var totalCount = 0;
  var countText = 0;
  var text = env.svg.select('.pie-labels').selectAll('text').data(pie(flag ? _config.data : _config.dataLegend));
  text.enter().append('text').attr('dy', '0.35em').style('opacity', 0).style('fill', function (d) {
    return d.data.color;
  }).text(function (d) {
    var percent = Number(d.value) / _d2.default.sum(flag ? _config.data : _config.dataLegend, function (d) {
      return d.value;
    }) * 100;
    if (percent < 4) {
      totalCount++;
    }
    return d.data.name + ':' + percent.toFixed(0) + '%';
  }).attr('transform', function (d) {
    var percent = Number(d.value) / _d2.default.sum(flag ? _config.data : _config.dataLegend, function (d) {
      return d.value;
    }) * 100;
    if (percent < 4) {
      var midCount = Math.floor(totalCount / 2);
      countText = countText + 1;
      if (countText <= midCount) {
        var posText = outerArc.centroid(d);
        posText[1] -= midAngle(d) < Math.PI ? -20 * countText : 20 * countText;
        return 'translate(' + posText + ')';
      } else {
        var num = countText - midCount;
        var _posText = outerArc.centroid(d);
        _posText[0] += 80;
        _posText[1] -= midAngle(d) < Math.PI ? -20 * num : 20 * num;
        return 'translate(' + _posText + ')';
      }
    } else {
      var pos = outerArc.centroid(d);
      pos[1] -= midAngle(d) < Math.PI ? -12 : 12;
      return 'translate(' + pos + ')';
    }
  }).style('text-anchor', function (d) {
    return midAngle(d) < Math.PI ? 'start' : 'end';
  }).transition().delay(function (d, i) {
    return arcAnimLineText + i * 250;
  }).duration(1000).style('opacity', 0.8);

  // 设置线条
  var countLine = 0;
  var polyline = env.svg.select('.pie-lines').selectAll('polyline').data(pie(flag ? _config.data : _config.dataLegend));
  polyline.enter().append('polyline').style('opacity', 0).attr('points', function (d) {
    return [arc.centroid(d), arc.centroid(d), arc.centroid(d)];
  }).transition().delay(function (d, i) {
    return i * 250;
  }).duration(1000).delay(function (d, i) {
    return arcAnimLineText + i * 250;
  }).attr('points', function (d) {
    var percent = Number(d.value) / _d2.default.sum(flag ? _config.data : _config.dataLegend, function (d) {
      return d.value;
    }) * 100;
    if (percent < 4) {
      var midCount = Math.floor(totalCount / 2);
      countLine = countLine + 1;
      if (countLine <= midCount) {
        var posLine = outerArc.centroid(d);
        posLine[0] -= midAngle(d) < Math.PI ? 8 : -8;
        posLine[1] -= midAngle(d) < Math.PI ? -20 * countLine : 20 * countLine;
        return [arc.centroid(d), posLine, [midAngle(d) < Math.PI ? posLine[0] + 6 : posLine[0] - 6, posLine[1]]];
      } else {
        var num = countLine - midCount;
        var _posLine = outerArc.centroid(d);
        _posLine[0] -= midAngle(d) < Math.PI ? 8 : -8;
        _posLine[1] -= midAngle(d) < Math.PI ? -20 * num : 20 * num;
        return [arc.centroid(d), _posLine, [midAngle(d) < Math.PI ? _posLine[0] - 6 : _posLine[0] + 6, _posLine[1]]];
      }
    } else {
      var pos = outerArc.centroid(d);
      pos[0] += midAngle(d) < Math.PI ? 60 : -60;
      return [arc.centroid(d), outerArc.centroid(d), pos];
    }
  }).style('stroke', function (d) {
    return d.data.color;
  }).style('opacity', 1.0);

  function midAngle(d) {
    return d.startAngle + (d.endAngle - d.startAngle) / 2;
  }
}; /**
    * Copyright 2016-present, Baifendian, Inc.
    * All rights reserved.
    *
    * This source code is licensed under the BSD-style license found in the
    * LICENSE file in the root directory of this source tree. An additional grant
    * of patent rights can be found in the PATENTS file in the same directory.
    */