'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _d = require('d3');

var _d2 = _interopRequireDefault(_d);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _class = function () {
  (0, _createClass3.default)(_class, [{
    key: 'getTickValues',


    /**
     * 获取Y轴 tickValues，nice 和 ticks 无法同时使用，只能手动划分
     */
    value: function getTickValues(yScale) {
      var max = yScale.invert(0);
      var values = [];
      var step = max / 5;
      for (var i = 0; i <= 5; i++) {
        values.push(step * i);
      }
      return values;
    }
  }]);

  function _class(config) {
    (0, _classCallCheck3.default)(this, _class);

    this.container = config.container;
    this.width = config.width;
    this.height = config.height;
    this.dataset = config.data;
    this.padding = config.padding;
    this.xScale = config.xScale;
    this.yScale = config.yScale;
    this.series = config.series;
    this.categories = config.categories;
    this.cols = config.cols;
    this.xLen = config.xLen;
  }

  (0, _createClass3.default)(_class, [{
    key: 'create',
    value: function create(config) {
      var container = this.container;
      var height = this.height;
      var width = this.width;
      var dataset = this.dataset;
      var padding = this.padding;
      var xScale = this.xScale;
      var yScale = this.yScale;
      var categories = this.categories;
      var _this = this;

      // 在 container 里添加一个 SVG 画布   
      var svg = _d2.default.select(container).append('svg').attr('width', width + padding.left + padding.right).attr('height', height + padding.top + padding.bottom);

      // 添加背景
      svg.append('g').append('rect').attr('x', 0).attr('y', 0).attr('width', width + padding.left + padding.right).attr('height', height + padding.top + padding.bottom).style('fill', '#FFF').style('stroke-width', 2).style('stroke', '#E7E7E7');

      var xItemLen = width / dataset.length;

      if (config.highBg) {
        var div = _d2.default.select(container).append('div').attr('class', 'tooltip').style({
          position: 'absolute',
          left: 0,
          top: 0,
          opacity: 0,
          'pointer-events': 'none'
        });

        dataset.map(function (data, index) {
          // 添加背景
          svg.append('g').append('rect').attr('x', index * xItemLen + padding.left + 1).attr('y', padding.top).attr('width', xItemLen - 1).attr('height', height).style('fill', '#FFF').style('stroke', '#FFF').on('mousemove', function () {
            _d2.default.select(this).attr('opacity', '0.8').style('fill', '#EBF4FF');

            div.transition().duration(200).style('opacity', .9);

            div.html(_this.getToolTipHtml(data, index)).style('left', _d2.default.event.offsetX + 15 + 'px').style('top', _d2.default.event.offsetY - div[0][0].clientHeight / 2 + 10 + 'px');
          }).on('mouseout', function () {
            _d2.default.select(this).attr('opacity', '1').style('fill', '#fff');
            div.transition().duration(200).style('opacity', 0);
          });
        });
      }
      // 定义x轴
      var xAxis = _d2.default.svg.axis().scale(xScale).orient('bottom');

      // 定义y轴
      var yAxis = _d2.default.svg.axis().scale(yScale).orient('left').tickValues(this.getTickValues(yScale)).tickFormat(function (d) {
        return _d2.default.format('s')(d);
      }).tickSize(-width, 0);
      this.yAxis = yAxis;

      // 添加x轴
      svg.append('g').attr('class', 'axis-x').attr('transform', 'translate(' + padding.left + ',' + (height + padding.top) + ')').style('fill', '#747779').call(xAxis);

      // 添加y轴
      svg.append('g').attr('class', 'axis-y').attr('transform', 'translate(' + padding.left + ',' + padding.top + ')').style('fill', '#747779').call(yAxis);

      // 定义横轴网格线
      var xInner = _d2.default.svg.axis().scale(xScale).tickSize(-height, 0).orient('bottom').ticks(dataset.length).tickValues(categories);

      // 添加横轴网格线
      svg.append('g').attr('class', 'inner_line').attr('transform', 'translate(' + (padding.left + xItemLen / 2) + ',' + (height + padding.top) + ')').call(xInner).selectAll('text').text('').attr('width', 10);
      this.svg = svg;
      return this;
    }
  }, {
    key: 'setYScale',
    value: function setYScale(yScale) {
      this.yAxis.tickValues(this.getTickValues(yScale));
      // 轴、曲线、曲面、标记点重绘
      this.svg.select('.axis-y').transition().call(this.yAxis);
    }
  }, {
    key: 'setSeries',
    value: function setSeries(series) {
      this.series = series;
    }
  }, {
    key: 'getToolTipHtml',
    value: function getToolTipHtml(data) {
      if (this.series.length == 0) {
        return '';
      }
      var tmp = '<tr><td style="color:{color};padding:0">{name}：</td><td style="padding:0"><b>{value}</b></td></tr>';
      var trs = '';

      for (var p in this.cols) {
        var name = this.cols[p];
        var value = data[p] || 0;
        var color = '';

        for (var i = 0; i < this.series.length; i++) {
          var serie = this.series[i];
          if (serie.key == p) {
            color = serie.color;
            break;
          }
        }
        if (color == '') {
          continue;
        }
        trs += tmp.replace('{name}', name).replace('{value}', value).replace('{color}', color);
      }
      return '<table>' + trs + '</table>';
    }
  }, {
    key: 'getSvg',
    value: function getSvg() {
      return this.svg;
    }
  }]);
  return _class;
}(); /**
      * Copyright 2016-present, Baifendian, Inc.
      * All rights reserved.
      *
      * This source code is licensed under the BSD-style license found in the
      * LICENSE file in the root directory of this source tree. An additional grant
      * of patent rights can be found in the PATENTS file in the same directory.
      */

exports.default = _class;