'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _reactUpdate = require('react-update');

var _reactUpdate2 = _interopRequireDefault(_reactUpdate);

var _get = require('lodash/get');

var _get2 = _interopRequireDefault(_get);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Copyright 2016-present, Baifendian, Inc.
 * All rights reserved.
 *
 * This source code is licensed under the BSD-style license found in the
 * LICENSE file in the root directory of this source tree. An additional grant
 * of patent rights can be found in the PATENTS file in the same directory.
 */

function updateItemCheckState(item, data, path, checked, indeterminate) {
  var newData = data;
  if (!!item.checked !== checked) {
    newData = (0, _reactUpdate2.default)(newData, 'set', [].concat((0, _toConsumableArray3.default)(path), ['checked']), checked);
  }
  if (!!item.indeterminate !== indeterminate) {
    newData = (0, _reactUpdate2.default)(newData, 'set', [].concat((0, _toConsumableArray3.default)(path), ['indeterminate']), indeterminate);
  }
  return newData;
}

function updateChildrenCheckedState(data, item, path, checked) {
  var newData = data;
  if (item && item.children) {
    path = [].concat((0, _toConsumableArray3.default)(path), ['children']);
    item.children.forEach(function (item, i) {
      var _path = [].concat((0, _toConsumableArray3.default)(path), [i]);
      newData = updateItemCheckState(item, newData, _path, checked, false);
      newData = updateChildrenCheckedState(newData, item, _path, checked);
    });
  }
  return newData;
}

var updateParentsCheckedState = function () {
  var isChecked = function isChecked(item) {
    var childrenCheckedLen = item.children.filter(function (item) {
      return item.checked;
    }).length;
    return childrenCheckedLen === item.children.length;
  };
  var isIndeterminate = function isIndeterminate(item) {
    return item.children.some(function (item) {
      return item.indeterminate || item.checked;
    });
  };
  return function updateParentCheckedState(data, path) {
    var newData = data;
    if (path.length) {
      var item = (0, _get2.default)(newData, path);
      var checked = isChecked(item);
      var indeterminate = !checked && isIndeterminate(item);
      newData = updateItemCheckState(item, newData, path, checked, indeterminate);
      if (newData !== data) {
        newData = updateParentCheckedState(newData, path.slice(0, -2));
      }
    }
    return newData;
  };
}();

function updateRelativeNodesCheckedState(data, item, path, checked) {
  var newData = updateChildrenCheckedState(data, item, path, checked);
  return updateParentsCheckedState(newData, path.slice(0, -2));
}

exports.default = function (data, checked, item, path, callback) {
  var newData = updateItemCheckState(item, data, path, checked, false);
  var newItem = (0, _get2.default)(newData, path);
  newData = updateRelativeNodesCheckedState(newData, newItem, path, checked);
  newItem = (0, _get2.default)(newData, path);
  callback(newData, newItem);
};